@@ -37,6 +37,8 @@
 type ('a, 'b) t = ('a, 'b) Hashtbl.t
 (** A Hashtable with keys of type 'a and values 'b *)
 
+type statistics = Hashtbl.statistics
+
 (**{6 Base operations}*)
 
 val create : int -> ('a, 'b) t
@@ -101,6 +103,12 @@ val copy : ('a, 'b) t -> ('a, 'b) t
 val clear : ('a, 'b) t -> unit
 (** Empty a hash table. *)
 
+val stats : ('a, 'b) t -> statistics
+(** [Hashtbl.stats tbl] returns statistics about the table [tbl]:
+    number of buckets, size of the biggest bucket, distribution of
+    buckets by size.
+    @since 4.00.0 and batteries 3.5.1 *)
+
 (**{6 Enumerations}*)
 
 val keys : ('a,'b) t -> 'a BatEnum.t
@@ -147,10 +155,13 @@ val find_default : ('a,'b) t -> 'a -> 'b -> 'b
 (** [Hashtbl.find_default tbl key default] finds a binding for [key],
     or return [default] if [key] is unbound in [tbl]. *)
 
-val find_option : ('a,'b) Hashtbl.t -> 'a -> 'b option
+val find_option : ('a,'b) t -> 'a -> 'b option
 (** Find a binding for the key, or return [None] if no
     value is found *)
 
+val exists : ('a -> 'b -> bool) -> ('a,'b) t -> bool
+(** Check if at least one key-value pair satisfies [p k v] *)
+
 val mem : ('a, 'b) t -> 'a -> bool
 (** [Hashtbl.mem tbl x] checks if [x] is bound in [tbl]. *)
 
@@ -185,6 +196,10 @@ val iter : ('a -> 'b -> unit) -> ('a, 'b) t -> unit
     they are passed to [f] in reverse order of introduction, that is,
     the most recent binding is passed first. *)
 
+val for_all : ('a -> 'b -> bool) -> ('a, 'b) t -> bool
+(** [Hashtbl.for_all p tbl] check if the predicate [p k v]
+    holds for all bindings currently in [tbl]. *)
+
 val fold : ('a -> 'b -> 'c -> 'c) -> ('a, 'b) t -> 'c -> 'c
 (** [Hashtbl.fold f tbl init] computes
     [(f kN dN ... (f k1 d1 (f k0 d0 init))...)],
@@ -331,6 +346,7 @@ sig
   val add : ('a, 'b) t -> key:'a -> data:'b -> unit
   val replace : ('a, 'b) t -> key:'a -> data:'b -> unit
   val iter : f:(key:'a -> data:'b -> unit) -> ('a, 'b) t -> unit
+  val for_all : f:(key:'a -> data:'b -> bool) -> ('a, 'b) t -> bool
   val map : f:(key:'a -> data:'b -> 'c) -> ('a, 'b) t -> ('a, 'c) t
   val map_inplace : f:(key:'a -> data:'b -> 'b) -> ('a,'b) t -> unit
   val filter : f:('a -> bool) -> ('key, 'a) t -> ('key, 'a) t
@@ -340,6 +356,7 @@ sig
   val filter_map : f:(key:'key -> data:'a -> 'b option) -> ('key, 'a) t -> ('key, 'b) t
   val filter_map_inplace : f:(key:'key -> data:'a -> 'a option) -> ('key, 'a) t -> unit
   val fold : f:(key:'a -> data:'b -> 'c -> 'c) -> ('a, 'b) t -> init:'c -> 'c
+  val exists : f:(key:'a -> data:'b -> bool) -> ('a, 'b) t -> bool
   val modify : key:'a -> f:('b -> 'b) -> ('a, 'b) t -> unit
   val modify_def : default:'b -> key:'a -> f:('b -> 'b) -> ('a, 'b) t -> unit
   val modify_opt : key:'a -> f:('b option -> 'b option) -> ('a, 'b) t -> unit
@@ -393,7 +410,9 @@ sig
   val replace : 'a t -> key -> 'a -> unit
   val mem : 'a t -> key -> bool
   val iter : (key -> 'a -> unit) -> 'a t -> unit
+  val for_all : (key -> 'a -> bool) -> 'a t -> bool
   val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
+  val exists : (key -> 'a -> bool) -> 'a t -> bool
   val map : (key -> 'b -> 'c) -> 'b t -> 'c t
   val map_inplace : (key -> 'a -> 'a) -> 'a t -> unit
   val filter : ('a -> bool) -> 'a t -> 'a t
@@ -408,7 +427,8 @@ sig
   val merge : (key -> 'a option -> 'b option -> 'c option) ->
              'a t -> 'b t -> 'c t
   val merge_all : (key -> 'a list -> 'b list -> 'c list) ->
-                  'a t -> 'b t -> 'c t
+    'a t -> 'b t -> 'c t
+  val stats : 'a t -> statistics
   val keys : 'a t -> key BatEnum.t
   val values : 'a t -> 'a BatEnum.t
   val enum : 'a t -> (key * 'a) BatEnum.t
@@ -469,6 +489,7 @@ sig
     val add : 'a t -> key:key -> data:'a -> unit
     val replace : 'a t -> key:key -> data:'a -> unit
     val iter : f:(key:key -> data:'a -> unit) -> 'a t -> unit
+    val for_all : f:(key:key -> data:'a -> bool) -> 'a t -> bool
     val map : f:(key:key -> data:'a -> 'b) -> 'a t -> 'b t
     val map_inplace : f:(key:key -> data:'a -> 'a) -> 'a t -> unit
     val filter : f:('a -> bool) -> 'a t -> 'a t
@@ -478,6 +499,7 @@ sig
     val filter_map : f:(key:key -> data:'a -> 'b option) -> 'a t -> 'b t
     val filter_map_inplace : f:(key:key -> data:'a -> 'a option) -> 'a t -> unit
     val fold : f:(key:key -> data:'a -> 'b -> 'b) -> 'a t -> init:'b -> 'b
+    val exists : f:(key:key -> data:'a -> bool) -> 'a t -> bool
     val modify : key:key -> f:('a -> 'a) -> 'a t -> unit
     val modify_def : default:'a -> key:key -> f:('a -> 'a) -> 'a t -> unit
     val modify_opt : key:key -> f:('a option -> 'a option) -> 'a t -> unit
@@ -550,6 +572,7 @@ sig
   val replace : ('a, 'b, [>`Write]) t -> 'a -> 'b -> unit
   val copy : ('a, 'b, [>`Read]) t -> ('a, 'b, _) t
   val clear : ('a, 'b, [>`Write]) t -> unit
+  val stats : ('a, 'b, _) t -> statistics
 
   (**{6 Searching}*)
 
@@ -557,6 +580,8 @@ sig
   val find_all : ('a, 'b, [>`Read]) t -> 'a -> 'b list
   val find_default : ('a, 'b, [>`Read]) t -> 'a -> 'b -> 'b
   val find_option : ('a, 'b, [>`Read]) t -> 'a -> 'b option
+  val exists : ('a -> 'b -> bool) -> ('a, 'b, [>`Read]) t -> bool
+
   val mem : ('a, 'b, [>`Read]) t -> 'a -> bool
 
   (*val exists : ('a,'b) t -> 'a -> bool
@@ -566,6 +591,7 @@ sig
   (**{6 Traversing}*)
 
   val iter : ('a -> 'b -> unit) -> ('a, 'b, [>`Read]) t -> unit
+  val for_all : ('a -> 'b -> bool) -> ('a, 'b, [>`Read]) t -> bool
   val fold : ('a -> 'b -> 'c -> 'c) -> ('a, 'b, [>`Read]) t -> 'c -> 'c
   val map : ('a -> 'b -> 'c) -> ('a, 'b, [>`Read]) t -> ('a, 'c, _) t
   val map_inplace : ('a -> 'b -> 'b) -> ('a, 'b, [>`Write]) t -> unit
@@ -613,6 +639,7 @@ sig
     val add : ('a, 'b, [>`Write]) t -> key:'a -> data:'b -> unit
     val replace : ('a, 'b, [>`Write]) t -> key:'a -> data:'b -> unit
     val iter : f:(key:'a -> data:'b -> unit) -> ('a, 'b, [>`Read]) t -> unit
+    val for_all : f:(key:'a -> data:'b -> bool) -> ('a, 'b, [>`Read]) t -> bool
     val map : f:(key:'a -> data:'b -> 'c) -> ('a, 'b, [>`Read]) t -> ('a, 'c, _) t
     val map_inplace : f:(key:'a -> data:'b -> 'b) -> ('a, 'b, [>`Write]) t -> unit
     val filter : f:('a -> bool) -> ('key, 'a, [>`Read]) t -> ('key, 'a, _) t
@@ -622,6 +649,7 @@ sig
     val filter_map : f:(key:'key -> data:'a -> 'b option) -> ('key, 'a, [>`Read]) t -> ('key, 'b, _) t
     val filter_map_inplace : f:(key:'key -> data:'a -> 'a option) -> ('key, 'a, [>`Write]) t -> unit
     val fold : f:(key:'a -> data:'b -> 'c -> 'c) -> ('a, 'b, [>`Read]) t -> init:'c -> 'c
+    val exists : f:(key:'a -> data:'b -> bool) -> ('a, 'b, [>`Read]) t -> bool
     val merge : f:('key -> 'a option -> 'b option -> 'c option) ->
                 left:('key, 'a, [>`Read]) t -> right:('key, 'b, [>`Read]) t -> ('key, 'c, _) t
     val merge_all : f:('key -> 'a list -> 'b list -> 'c list) ->
