@@ -19,6 +19,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
 
+##V>=5##module Pervasives := Stdlib
+
 (** Association tables over ordered types.
 
     This module implements applicative association tables, also known as
@@ -73,7 +75,9 @@ sig
   type key
   (** The type of the map keys. *)
 
-  type (+'a) t
+  type +
+##V>=4.12## !
+  'a t
   (** The type of maps from type [key] to type ['a]. *)
 
   val empty: 'a t
@@ -88,16 +92,39 @@ sig
   val add: key -> 'a -> 'a t -> 'a t
   (** [add x y m] returns a map containing the same bindings as
       [m], plus a binding of [x] to [y]. If [x] was already bound
-      in [m], its previous binding disappears. *)
+      in [m], its previous binding disappears. 
+      If [x] was already bound to some [z] that is physically equal
+      to [y], then the returned map is physically equal to [m].
 
+      @before 3.3.0 or OCaml 4.03 physical equality was not ensured. *)
+
+  val update_stdlib : key -> ('a option -> 'a option) -> 'a t -> 'a t
+  (** [update_stdlib k f m] returns a map containing the same bindings as [m],
+      except [k] has a new binding as determined by [f]:
+      First, calculate [y] as [f (find_opt k m)].
+      If [y = Some v] then [k] will be bound to [v] in the resulting map.
+      Else [k] will not be bound in the resulting map.
+      If [v] is physically equal to the value of the previous binding of [k] in [m],
+      then the returned map will be physically equal to [m].
+
+      This function does the same thing as [update] in the stdlib, but has a
+      different name for backwards compatibility reasons.
+
+      @since 3.3.0 *)
+
+  (* TODO: maybe deprecate this function to re-gain compatibility with stdlib? *)
   val update: key -> key -> 'a -> 'a t -> 'a t
   (** [update k1 k2 v2 m] replace the previous binding of [k1] in [m] by
       [k2] associated to [v2].
       This is equivalent to [add k2 v2 (remove k1) m], but more efficient
       in the case where [k1] and [k2] have the same key ordering.
+      If [k1] and [k2] have the same key ordering and [v2] is physically
+      equal to the value [k1] is bound to in [m] then the returned map will
+      be physically equal to [m]
       @raise Not_found if [k1] is not bound in [m].
-      @since 2.4.0 *)
-
+      @since 2.4.0 
+      @before 3.3.0 physical equality was not ensured.  *)
+    
   val find: key -> 'a t -> 'a
   (** [find x m] returns the current binding of [x] in [m],
       or raises [Not_found] if no such binding exists. *)
@@ -110,9 +137,51 @@ sig
   (** [find_default d x m] returns the current binding of [x] in [m],
       or the default value [d] if no such binding exists. *)
 
+  val find_first: (key -> bool) -> 'a t -> key * 'a
+  (** [find_first f m] returns the first binding [(k, v)] for which [f k] is true
+      or raises [Not_found] if there is no such binding.
+      [f] must be monotonically increasing,
+      i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+    
+      @since 3.3.0 *)
+
+  val find_first_opt: (key -> bool) -> 'a t -> (key * 'a) option
+  (** [find_first_opt f m] returns [Some (k, v)] for the first binding [(k, v)]
+      for which [f k] is true or returns [None] if there is no such binding.
+      [f] must be monotonically increasing,
+      i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+    
+      @since 3.3.0 *)
+
+  val find_last: (key -> bool) -> 'a t -> key * 'a
+  (** [find_last f m] returns the last binding [(k, v)] for which [f k] is true
+      or raises [Not_found] if there is no such binding.
+      [f] must be monotonically decreasing,
+      i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true.  
+    
+      @since 3.3.0*)
+
+  val find_last_opt: (key -> bool) -> 'a t -> (key * 'a) option
+  (** [find_last_opt f m] returns [Some (k, v)] for the last binding [(k, v)]
+      for which [f k] is true or returns [None] if there is no such binding.
+      [f] must be monotonically decreasing,
+      i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+    
+      @since 3.3.0 *)
+
   val remove: key -> 'a t -> 'a t
   (** [remove x m] returns a map containing the same bindings as
-      [m], except for [x] which is unbound in the returned map. *)
+      [m], except for [x] which is unbound in the returned map.
+      The returned map is physically equal  to the passed one if [x] was
+      already unbound. 
+  
+      @before 3.3.0 or OCaml 4.03 physical equality was not ensured. *)
+
+  val remove_exn: key -> 'a t -> 'a t
+  (** [remove_exn x m] behaves like [remove x m] except that it raises
+      an exception if [x] is unbound in [m].
+      @raise Not_found if [x] is unbound in [m]
+      @since 3.2.0 *)
 
   val modify: key -> ('a -> 'a) -> 'a t -> 'a t
   (** [modify k f m] replaces the previous binding for [k] with [f] applied to
@@ -190,7 +259,11 @@ sig
   (** [filter f m] returns a map where only the [(key, value)] pairs of [m]
       such that [f key value = true] remain. The bindings are passed to
       [f] in increasing order with respect to the ordering over the type
-      of the keys. *)
+      of the keys. 
+      If [f] returns [true] for all bindings of [m] the returned map is physically 
+      equal to [m].
+      
+      @before 3.3.0 or OCaml 4.03 physical equality was not ensured. *)
 
   val filter_map: (key -> 'a -> 'b option) -> 'a t -> 'b t
   (** [filter_map f m] combines the features of [filter] and
@@ -219,14 +292,28 @@ sig
       The returned enumeration is sorted in increasing key order. *)
 
   val min_binding : 'a t -> (key * 'a)
-  (** Return the [(key, value)] pair with the smallest key. *)
+  (** Return the [(key, value)] pair with the smallest key.
+      @raise Not_found if the map is empty.  *)
+    
+  val min_binding_opt : 'a t -> (key * 'a) option
+  (** Return [Some (key, value)] for the [key, value] pair with 
+      the smallest key, or [None] if the map is empty.
+    
+      @since 3.3.0  *)
 
   val pop_min_binding : 'a t -> (key * 'a) * 'a t
   (** Return the [(key, value)] pair with the smallest key
       along with the rest of the map. *)
 
   val max_binding : 'a t -> (key * 'a)
-  (** Return the [(key, value)] pair with the largest key. *)
+  (** Return the [(key, value)] pair with the largest key. 
+      Raises Not_found if the map is empty.  *)
+    
+  val max_binding_opt :  'a t -> (key * 'a) option
+  (** Return [Some (key, value)] for the [key, value] pair with 
+      the largest key, or [None] if the map is empty. 
+    
+      @since 3.3.0 *)
 
   val pop_max_binding : 'a t -> (key * 'a) * 'a t
   (** Return the ([key, value]) pair with the largest key
@@ -244,6 +331,14 @@ sig
       @raise Not_found if the map is empty
   *)
 
+  val choose_opt : 'a t -> (key * 'a) option
+  (** Return [Some (k, v)] for one binding [(k, v)] of the given map,
+     if the map is not empty. Else, return None.  Which binding is
+     chosen is unspecified, but equal bindings will be chosen for
+     equal maps.
+
+     @since 3.3.0 *)
+
   val any : 'a t -> (key * 'a)
   (** Return one binding of the given map.
       The difference with choose is that there is no guarantee that equals
@@ -313,6 +408,48 @@ sig
       value, is determined with the function [f].
   *)
 
+  val union:
+    (key -> 'a -> 'a -> 'a option) -> 'a t -> 'a t -> 'a t
+  (** [union f m1 m2] computes a map whose keys are a subset of the keys of 
+      [m1] and of [m2]. When the same binding is defined in both arguments,
+      the function f is used to combine them.
+      This function is similar to [merge], except [f] is only called if a key
+      is present in both [m1] and [m2]. If a key is present in either [m1]
+      or [m2] but not in both, it (and the corresponding value) will be 
+      present in the resulting map.
+    
+      @since 3.3.0 *)
+
+  val to_seq : 'a t -> (key * 'a) BatSeq.t
+  (** Iterate on the whole map, in ascending order of keys.
+
+      @since 3.3.0  *)
+
+  val to_rev_seq : 'a t -> (key * 'a) BatSeq.t
+  (** Iterate on the whole map, in descending order of keys.
+
+      @since 3.3.0  *)
+    
+  val to_seq_from :  key -> 'a t -> (key * 'a) BatSeq.t
+  (** [to_seq_from k m] iterates on a subset of the bindings in [m], 
+      namely those bindings greater or equal to [k], in ascending order. 
+    
+      @since 3.3.0 *)
+    
+  val add_seq : (key * 'a) BatSeq.t -> 'a t -> 'a t
+  (** add the given bindings to the map, in order. 
+    
+      @since 3.3.0  *)
+    
+  val of_seq : (key * 'a) BatSeq.t -> 'a t
+  (** build a map from the given bindings 
+    
+      @since 3.3.0 *)
+
+  val to_list : 'a t -> (key * 'a) list
+  val of_list : (key * 'a) list -> 'a t
+  val add_to_list: key -> 'a -> 'a list t -> 'a list t
+
   (** {6 Boilerplate code}*)
 
   (** {7 Printing}*)
@@ -403,7 +540,7 @@ module String : S with type key = string
     function, it is recommended to use the functorized maps provided
     by {!Make}.  *)
 
-type ('a, 'b) t
+type (+'a, +'b) t
 
 val empty : ('a, 'b) t
 (** The empty map, using [compare] as key comparison function. *)
@@ -420,15 +557,37 @@ val cardinal: ('a, 'b) t -> int
 val add : 'a -> 'b -> ('a, 'b) t -> ('a, 'b) t
 (** [add x y m] returns a map containing the same bindings as
     [m], plus a binding of [x] to [y]. If [x] was already bound
-    in [m], its previous binding disappears. *)
+    in [m], its previous binding disappears. 
+    If [x] was already bound to some [z] that is physically equal
+    to [y], then the returned map is physically equal to [m].
+
+    @before 3.3.0 or OCaml 4.03 physical equality was not ensured. *)
 
 val update: 'a -> 'a -> 'b -> ('a, 'b) t -> ('a, 'b) t
 (** [update k1 k2 v2 m] replace the previous binding of [k1] in [m] by
     [k2] associated to [v2].
     This is equivalent to [add k2 v2 (remove k1) m], but more efficient
     in the case where [k1] and [k2] have the same key ordering.
+    If [k1] and [k2] have the same key ordering and [v2] is physically
+    equal to the value [k1] is bound to in [m] then the returned map will
+    be physically equal to [m]
     @raise Not_found if [k1] is not bound in [m].
-    @since 2.4.0 *)
+    @since 2.4.0 
+    @before 3.3.0 physical equality was not ensured. *)
+
+val update_stdlib : 'a -> ('b option -> 'b option) -> ('a, 'b) t -> ('a, 'b) t
+(** [update_stdlib k f m] returns a map containing the same bindings as [m],
+    except [k] has a new binding as determined by [f]:
+    First, calculate [y] as [f (find_opt k m)].
+    If [y = Some v] then [k] will be bound to [v] in the resulting map.
+    Else [k] will not be bound in the resulting map.
+
+    If [v] is physically equal to the value of the previous binding of [k] in [m],
+    then the returned map will be physically equal to [m].
+
+    This function does the same thing as [update] in the stdlib, but has a
+    different name for backwards compatibility reasons.
+    @since 3.3.0 *)
 
 val find : 'a -> ('a, 'b) t -> 'b
 (** [find x m] returns the current binding of [x] in [m],
@@ -442,9 +601,51 @@ val find_default : 'b -> 'a -> ('a, 'b) t -> 'b
 (** [find_default d x m] returns the current binding of [x] in [m],
     or the default value [d] if no such binding exists. *)
 
+val find_first: ('a -> bool) -> ('a, 'b) t -> 'a * 'b
+(** [find_first f m] returns the first binding [(k, v)] for which [f k] is true
+    or raises [Not_found] if there is no such binding.
+    [f] must be monotonically increasing,
+    i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+    
+    @since 3.3.0 *)
+
+val find_first_opt: ('a -> bool) -> ('a, 'b) t -> ('a * 'b) option
+(** [find_first_opt f m] returns [Some (k, v)] for the first binding [(k, v)]
+    for which [f k] is true or returns [None] if there is no such binding.
+    [f] must be monotonically increasing,
+    i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true.  
+    
+    @since 3.3.0 *)
+
+val find_last: ('a -> bool) -> ('a, 'b) t -> 'a * 'b
+(** [find_last f m] returns the last binding [(k, v)] for which [f k] is true
+    or raises [Not_found] if there is no such binding.
+    [f] must be monotonically decreasing,
+    i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+    
+    @since 3.3.0 *)
+
+val find_last_opt: ('a -> bool) -> ('a, 'b) t -> ('a * 'b) option
+(** [find_last_opt f m] returns [Some (k, v)] for the last binding [(k, v)]
+    for which [f k] is true or returns [None] if there is no such binding.
+    [f] must be monotonically decreasing,
+    i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+    
+    @since 3.3.0 *)
+
 val remove : 'a -> ('a, 'b) t -> ('a, 'b) t
 (** [remove x m] returns a map containing the same bindings as
-    [m], except for [x] which is unbound in the returned map. *)
+    [m], except for [x] which is unbound in the returned map.
+    The returned map is physically equal  to the passed one if [x] was
+    already unbound. 
+
+    @before 3.3.0 or OCaml 4.03 physical equality was not ensured. *)
+
+val remove_exn: 'a -> ('a, 'b) t -> ('a, 'b) t
+(** [remove_exn x m] behaves like [remove x m] except that it raises
+    an exception if [x] is unbound in [m].
+    @raise Not_found if [x] is unbound in [m]
+    @since 3.2.0 *)
 
 val mem : 'a -> ('a, 'b) t -> bool
 (** [mem x m] returns [true] if [m] contains a binding for [x],
@@ -496,10 +697,14 @@ val filterv: ('a -> bool) -> ('key, 'a) t -> ('key, 'a) t
    type of the keys. *)
 
 val filter: ('key -> 'a -> bool) -> ('key, 'a) t -> ('key, 'a) t
-(**[filter f m] returns a map where only the [(key, value)] pairs
-   [key], [a] of [m] such that [f key a = true] remain. The
-   bindings are passed to [f] in increasing order with respect
-   to the ordering over the type of the keys. *)
+(** [filter f m] returns a map where only the [(key, value)] pairs of [m]
+    such that [f key value = true] remain. The bindings are passed to
+    [f] in increasing order with respect to the ordering over the type
+    of the keys. 
+    If [f] returns [true] for all bindings of [m] the returned map is physically 
+    equal to [m].
+    
+    @before 3.3.0 or OCaml 4.03 physical equality was not ensured. *)
 
 val filter_map: ('key -> 'a -> 'b option) -> ('key, 'a) t -> ('key, 'b) t
 (** [filter_map f m] combines the features of [filter] and
@@ -516,6 +721,15 @@ val choose : ('key, 'a) t -> ('key * 'a)
     @raise Not_found if the map is empty
 *)
 
+val choose_opt : ('key, 'a) t -> ('key * 'a) option
+(** Return [Some (k, v)] for one binding [(k, v)] of the given map,
+    if the map is not empty. Else, return None.
+    Which binding is chosen is unspecified, but equal bindings will be
+    chosen for equal maps.
+
+    @since 3.3.0 *)
+
+  
 val any : ('key, 'a) t -> ('key * 'a)
 (** Return one binding of the given map.
     The difference with choose is that there is no guarantee that equals
@@ -535,13 +749,27 @@ val split : 'key -> ('key, 'a) t -> (('key, 'a) t * 'a option * ('key, 'a) t)
 *)
 
 val min_binding : ('key, 'a) t -> ('key * 'a)
-(** Returns the binding with the smallest key. *)
+(** Returns the binding with the smallest key.
+    Raises Not_found if the map is empty.  *)
+    
+val min_binding_opt : ('key, 'a) t -> ('key * 'a) option
+(** Return [Some (key, value)] for the [key, value] pair with 
+    the smallest key, or [None] if the map is empty.
+    
+    @since 3.3.0  *)
 
 val pop_min_binding : ('key, 'a) t -> ('key * 'a) * ('key, 'a) t
 (** Returns the binding with the smallest key along with the rest of the map. *)
 
 val max_binding : ('key, 'a) t -> ('key * 'a)
-(** Returns the binding with the largest key. *)
+(** Return the [(key, value)] pair with the largest key. 
+    Raises Not_found if the map is empty.  *)
+    
+val max_binding_opt :  ('key, 'a) t -> ('key * 'a) option
+(** Return [Some (key, value)] for the [key, value] pair with 
+    the largest key, or [None] if the map is empty. 
+    
+    @since 3.3.0 *)
 
 val pop_max_binding : ('key, 'a) t -> ('key * 'a) * ('key, 'a) t
 (** Returns the binding with the largest key along with the rest of the map. *)
@@ -617,6 +845,46 @@ val union : ('a, 'b) t -> ('a, 'b) t -> ('a, 'b) t
     [m1]'s. Equivalent to [foldi add m2 m1].
     The resulting map uses the comparison function of [m1]. *)
 
+val union_stdlib:
+  ('key -> 'a -> 'a -> 'a option) -> ('key, 'a) t -> ('key, 'a) t -> ('key, 'a) t
+(** [union_stdlib f m1 m2] computes a map whose keys are a subset of the keys of 
+    [m1] and of [m2]. When the same binding is defined in both arguments,
+    the function f is used to combine them.
+    This function is similar to [merge], except [f] is only called if a key
+    is present in both [m1] and [m2]. If a key is present in either [m1]
+    or [m2] but not in both, it (and the corresponding value) will be 
+    present in the resulting map.
+
+    This is the union method from the stdlib map, renamed for backwards compatibility.
+    
+    @since 3.3.0 *)
+
+val to_seq : ('key, 'a) t -> ('key * 'a) BatSeq.t
+(** Iterate on the whole map, in ascending order of keys.
+    
+    @since 3.3.0  *)
+  
+val to_rev_seq : ('key, 'a) t -> ('key * 'a) BatSeq.t
+(** Iterate on the whole map, in descending order of keys.
+
+    @since 3.3.0  *)
+
+val to_seq_from :  'key -> ('key, 'a) t -> ('key * 'a) BatSeq.t
+(** [to_seq_from k m] iterates on a subset of the bindings in [m], 
+    namely those bindings greater or equal to [k], in ascending order. 
+    
+    @since 3.3.0 *)
+  
+val add_seq : ('key * 'a) BatSeq.t -> ('key, 'a) t -> ('key, 'a) t
+(** add the given bindings to the map, in order. 
+    
+    @since 3.3.0  *)
+  
+val of_seq : ('key * 'a) BatSeq.t -> ('key, 'a) t
+(** build a map from the given bindings 
+    
+    @since 3.3.0 *)
+
 val diff :  ('a, 'b) t -> ('a, 'b) t -> ('a, 'b) t
 (** [diff m1 m2] removes all bindings of keys found in [m2] from [m1],
     using the comparison function of [m1]. Equivalent to
@@ -711,7 +979,7 @@ module PMap : sig
       functor for additional safety.
   *)
 
-  type ('a, 'b) t
+  type ('a, +'b) t
 
   val empty : ('a, 'b) t
   (** The empty map, using [compare] as key comparison function. *)
@@ -734,16 +1002,39 @@ module PMap : sig
   val add : 'a -> 'b -> ('a, 'b) t -> ('a, 'b) t
   (** [add x y m] returns a map containing the same bindings as
       [m], plus a binding of [x] to [y]. If [x] was already bound
-      in [m], its previous binding disappears. *)
+      in [m], its previous binding disappears. 
+      
+      If [x] was already bound to some [z] that is physically equal
+      to [y], then the returned map is physically equal to [m].
+
+      @before 3.3.0 or OCaml 4.03 physical equality was not ensured. *)
 
   val update : 'a -> 'a -> 'b -> ('a, 'b) t -> ('a, 'b) t
   (** [update k1 k2 v2 m] replace the previous binding of [k1] in [m] by
       [k2] associated to [v2].
       This is equivalent to [add k2 v2 (remove k1) m], but more efficient
       in the case where [k1] and [k2] have the same key ordering.
+      If [k1] and [k2] have the same key ordering and [v2] is physically
+      equal to the value [k1] is bound to in [m] then the returned map will
+      be physically equal to [m]
       @raise Not_found if [k1] is not bound in [m].
-      @since 2.4.0 *)
+      @since 2.4.0 
+      @before 3.3.0 physical equality was not ensured.  *)
 
+  val update_stdlib : 'a -> ('b option -> 'b option) -> ('a, 'b) t -> ('a, 'b) t
+  (** [update_stdlib k f m] returns a map containing the same bindings as [m],
+      except [k] has a new binding as determined by [f]:
+      First, calculate [y] as [f (find_opt k m)].
+      If [y = Some v] then [k] will be bound to [v] in the resulting map.
+      Else [k] will not be bound in the resulting map.
+      If [v] is physically equal to the value of the previous binding of [k] in [m],
+      then the returned map will be physically equal to [m].
+
+      This function does the same thing as [update] in the stdlib, but has a
+      different name for backwards compatibility reasons.
+
+      @since 3.3.0 *)
+  
   val find : 'a -> ('a, 'b) t -> 'b
   (** [find x m] returns the current binding of [x] in [m],
       or raises [Not_found] if no such binding exists. *)
@@ -752,9 +1043,51 @@ module PMap : sig
   (** [find_default d x m] returns the current binding of [x] in [m],
       or the default value [d] if no such binding exists. *)
 
+  val find_first: ('a -> bool) -> ('a, 'b) t -> 'a * 'b
+  (** [find_first f m] returns the first binding [(k, v)] for which [f k] is true
+      or raises [Not_found] if there is no such binding.
+      [f] must be monotonically increasing,
+      i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+    
+      @since 3.3.0 *)
+
+  val find_first_opt: ('a -> bool) -> ('a, 'b) t -> ('a * 'b) option
+  (** [find_first_opt f m] returns [Some (k, v)] for the first binding [(k, v)]
+      for which [f k] is true or returns [None] if there is no such binding.
+      [f] must be monotonically increasing,
+      i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+    
+      @since 3.3.0 *)
+
+  val find_last: ('a -> bool) -> ('a, 'b) t -> 'a * 'b
+  (** [find_last f m] returns the last binding [(k, v)] for which [f k] is true
+      or raises [Not_found] if there is no such binding.
+      [f] must be monotonically decreasing,
+      i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+    
+      @since 3.3.0 *)
+
+  val find_last_opt: ('a -> bool) -> ('a, 'b) t -> ('a * 'b) option
+  (** [find_last_opt f m] returns [Some (k, v)] for the last binding [(k, v)]
+      for which [f k] is true or returns [None] if there is no such binding.
+      [f] must be monotonically decreasing,
+      i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+    
+      @since 3.3.0 *)
+
   val remove : 'a -> ('a, 'b) t -> ('a, 'b) t
   (** [remove x m] returns a map containing the same bindings as
-      [m], except for [x] which is unbound in the returned map. *)
+      [m], except for [x] which is unbound in the returned map.
+      The returned map is physically equal  to the passed one if [x] was
+      already unbound. 
+  
+      @before 3.3.0 or OCaml 4.03 physical equality was not ensured. *)
+
+  val remove_exn : 'a -> ('a, 'b) t -> ('a, 'b) t
+  (** [remove_exn x m] behaves like [remove x m] except that it raises
+      an exception if [x] is unbound in [m].
+      @raise Not_found if [x] is unbound in [m]
+      @since 3.2.0 *)
 
   val mem : 'a -> ('a, 'b) t -> bool
   (** [mem x m] returns [true] if [m] contains a binding for [x],
@@ -806,10 +1139,14 @@ module PMap : sig
      type of the keys. *)
 
   val filter: ('key -> 'a -> bool) -> ('key, 'a) t -> ('key, 'a) t
-  (**[filter f m] returns a map where only the [(key, value)] pairs
-     [key], [a] of [m] such that [f key a = true] remain. The
-     bindings are passed to [f] in increasing order with respect
-     to the ordering over the type of the keys. *)
+  (** [filter f m] returns a map where only the [(key, value)] pairs of [m]
+      such that [f key value = true] remain. The bindings are passed to
+      [f] in increasing order with respect to the ordering over the type
+      of the keys. 
+      If [f] returns [true] for all bindings of [m] the returned map is physically 
+      equal to [m].
+      
+      @before 3.3.0 or OCaml 4.03 physical equality was not ensured. *)
 
   val filter_map: ('key -> 'a -> 'b option) -> ('key, 'a) t -> ('key, 'b) t
   (** [filter_map f m] combines the features of [filter] and
@@ -824,6 +1161,14 @@ module PMap : sig
       Which binding is chosen is unspecified, but equal bindings will be chosen
       for equal maps.
       @raise Not_found if the map is empty. *)
+    
+  val choose_opt : ('key, 'a) t -> ('key * 'a) option
+  (** Return [Some (k, v)] for one binding [(k, v)] of the given map,
+     if the map is not empty. Else, return None.  Which binding is
+     chosen is unspecified, but equal bindings will be chosen for
+     equal maps.
+
+     @since 3.3.0 *)
 
   val any : ('key, 'a) t -> ('key * 'a)
   (** Return one binding of the given map.
@@ -845,6 +1190,12 @@ module PMap : sig
 
   val min_binding : ('key, 'a) t -> ('key * 'a)
   (** Returns the binding with the smallest key. *)
+    
+  val min_binding_opt : ('key, 'a) t -> ('key * 'a) option
+  (** Return [Some (key, value)] for the [key, value] pair with 
+      the smallest key, or [None] if the map is empty.
+    
+      @since 3.3.0  *)
 
   val pop_min_binding : ('key, 'a) t -> ('key * 'a) * ('key, 'a) t
   (** Return the binding with the smallest key along with the rest of the map. *)
@@ -852,6 +1203,12 @@ module PMap : sig
   val max_binding : ('key, 'a) t -> ('key * 'a)
   (** Returns the binding with the largest key. *)
 
+  val max_binding_opt :  ('key, 'a) t -> ('key * 'a) option
+  (** Return [Some (key, value)] for the [key, value] pair with 
+      the largest key, or [None] if the map is empty. 
+    
+      @since 3.3.0 *)
+
   val pop_max_binding : ('key, 'a) t -> ('key * 'a) * ('key, 'a) t
   (** Return the binding with the largest key along with the rest of the map. *)
 
@@ -954,6 +1311,47 @@ module PMap : sig
       comparison function of its first parameter, but which ['b option]
       elements are passed to the function is unspecified. *)
 
+  val union_stdlib:
+    ('key -> 'a -> 'a -> 'a option) -> ('key, 'a) t -> ('key, 'a) t -> ('key, 'a) t
+  (** [union f m1 m2] computes a map whose keys are a subset of the keys of 
+      [m1] and of [m2]. When the same binding is defined in both arguments,
+      the function f is used to combine them.
+      This function is similar to [merge], except [f] is only called if a key
+      is present in both [m1] and [m2]. If a key is present in either [m1]
+      or [m2] but not in both, it (and the corresponding value) will be 
+      present in the resulting map.
+
+      This is the union method from the stdlib map, renamed for backwards compatibility.
+
+      @since 3.3.0 *)
+
+  val to_seq : ('key, 'a) t -> ('key * 'a) BatSeq.t
+  (** Iterate on the whole map, in ascending order of keys.
+
+      @since 3.3.0  *)
+    
+  val to_rev_seq : ('key, 'a) t -> ('key * 'a) BatSeq.t
+  (** Iterate on the whole map, in descending order of keys.
+  
+      @since 3.3.0  *)
+
+  val to_seq_from : 'key -> ('key, 'a) t -> ('key * 'a) BatSeq.t
+  (** [to_seq_from k m] iterates on a subset of the bindings in [m], 
+      namely those bindings greater or equal to [k], in ascending order. 
+    
+      @since 3.3.0 *)
+    
+  val add_seq : ('key * 'a) BatSeq.t -> ('key, 'a) t -> ('key, 'a) t
+  (** add the given bindings to the map, in order. 
+    
+      @since 3.3.0  *)
+    
+  val of_seq : ?cmp:('key -> 'key -> int) -> ('key * 'a) BatSeq.t -> ('key, 'a) t
+  (** build a map from the given bindings 
+    
+      @since 3.3.0 *)
+
+
   val compare: ('b -> 'b -> int) -> ('a,'b) t -> ('a, 'b) t -> int
   val equal : ('b -> 'b -> bool) -> ('a,'b) t -> ('a, 'b) t -> bool
   (** Construct a comparison or equality function for maps based on a
